@@ -406,6 +406,28 @@ struct aws_credentials_provider_chain_default_options {
     struct aws_client_bootstrap *bootstrap;
 };
 
+typedef int(aws_credentials_provider_delegate_get_credentials_fn)(
+    void *delegate_user_data,
+    aws_on_get_credentials_callback_fn callback,
+    void *callback_user_data);
+
+/**
+ * Configuration options for the delegate credentials provider.
+ */
+struct aws_credentials_provider_delegate_options {
+    struct aws_credentials_provider_shutdown_options shutdown_options;
+
+    /**
+     * Delegated get_credentials() callback.
+     */
+    aws_credentials_provider_delegate_get_credentials_fn *get_credentials;
+
+    /**
+     * User data for delegated callbacks.
+     */
+    void *delegate_user_data;
+};
+
 AWS_EXTERN_C_BEGIN
 
 /*
@@ -705,6 +727,19 @@ struct aws_credentials_provider *aws_credentials_provider_new_process(
     struct aws_allocator *allocator,
     const struct aws_credentials_provider_process_options *options);
 
+/**
+ * Create a credentials provider depends on provided vtable to fetch the credentials.
+ *
+ * @param allocator memory allocator to use for all memory allocation
+ * @param options provider-specific configuration options
+ *
+ * @return the newly-constructed credentials provider, or NULL if an error occurred.
+ */
+AWS_AUTH_API
+struct aws_credentials_provider *aws_credentials_provider_new_delegate(
+    struct aws_allocator *allocator,
+    struct aws_credentials_provider_delegate_options *options);
+
 /**
  * Creates the default provider chain used by most AWS SDKs.
  *
