@@ -6,12 +6,15 @@
 #include <aws/common/common.h>
 #include <aws/common/logging.h>
 #include <aws/common/math.h>
+#include <aws/common/private/dlloads.h>
 
 #include <stdarg.h>
 #include <stdlib.h>
 
 #ifdef _WIN32
 #    include <Windows.h>
+#else
+#    include <dlfcn.h>
 #endif
 
 #ifdef __MACH__
@@ -24,6 +27,14 @@
 #    pragma warning(disable : 4100)
 #endif
 
+long (*g_set_mempolicy_ptr)(int, const unsigned long *, unsigned long) = NULL;
+int (*g_numa_available_ptr)(void) = NULL;
+int (*g_numa_num_configured_nodes_ptr)(void) = NULL;
+int (*g_numa_num_possible_cpus_ptr)(void) = NULL;
+int (*g_numa_node_of_cpu_ptr)(int cpu) = NULL;
+
+void *g_libnuma_handle = NULL;
+
 void aws_secure_zero(void *pBuf, size_t bufsize) {
 #if defined(_WIN32)
     SecureZeroMemory(pBuf, bufsize);
@@ -237,6 +248,7 @@ static struct aws_log_subject_info s_common_log_subject_infos[] = {
         AWS_LS_COMMON_TASK_SCHEDULER,
         "task-scheduler",
         "Subject for task scheduler or task specific logging."),
+    DEFINE_LOG_SUBJECT_INFO(AWS_LS_COMMON_THREAD, "thread", "Subject for logging thread related functions."),
     DEFINE_LOG_SUBJECT_INFO(AWS_LS_COMMON_XML_PARSER, "xml-parser", "Subject for xml parser specific logging."),
     DEFINE_LOG_SUBJECT_INFO(AWS_LS_COMMON_MEMTRACE, "memtrace", "Output from the aws_mem_trace_dump function"),
 };
@@ -255,6 +267,66 @@ void aws_common_library_init(struct aws_allocator *allocator) {
         s_common_library_initialized = true;
         aws_register_error_info(&s_list);
         aws_register_log_subject_info_list(&s_common_log_subject_list);
+
+/* NUMA is funky and we can't rely on libnuma.so being available. We also don't want to take a hard dependency on it,
+ * try and load it if we can. */
+#if !defined(_WIN32) && !defined(WIN32)
+        /* libnuma defines set_mempolicy() as a WEAK symbol. Loading into the global symbol table overwrites symbols and
+           assumptions due to the way loaders and dlload are often implemented and those symbols are defined by things
+           like libpthread.so on some unix distros. Sorry about the memory usage here, but it's our only safe choice.
+           Also, please don't do numa configurations if memory is your economic bottlneck. */
+        g_libnuma_handle = dlopen("libnuma.so", RTLD_LOCAL);
+
+        /* turns out so versioning is really inconsistent these days */
+        if (!g_libnuma_handle) {
+            g_libnuma_handle = dlopen("libnuma.so.1", RTLD_LOCAL);
+        }
+
+        if (!g_libnuma_handle) {
+            g_libnuma_handle = dlopen("libnuma.so.2", RTLD_LOCAL);
+        }
+
+        if (g_libnuma_handle) {
+            AWS_LOGF_INFO(AWS_LS_COMMON_GENERAL, "static: libnuma.so loaded");
+            *(void **)(&g_set_mempolicy_ptr) = dlsym(g_libnuma_handle, "set_mempolicy");
+            if (g_set_mempolicy_ptr) {
+                AWS_LOGF_INFO(AWS_LS_COMMON_GENERAL, "static: set_mempolicy() loaded");
+            } else {
+                AWS_LOGF_INFO(AWS_LS_COMMON_GENERAL, "static: set_mempolicy() failed to load");
+            }
+
+            *(void **)(&g_numa_available_ptr) = dlsym(g_libnuma_handle, "numa_available");
+            if (g_numa_available_ptr) {
+                AWS_LOGF_INFO(AWS_LS_COMMON_GENERAL, "static: numa_available() loaded");
+            } else {
+                AWS_LOGF_INFO(AWS_LS_COMMON_GENERAL, "static: numa_available() failed to load");
+            }
+
+            *(void **)(&g_numa_num_configured_nodes_ptr) = dlsym(g_libnuma_handle, "numa_num_configured_nodes");
+            if (g_numa_num_configured_nodes_ptr) {
+                AWS_LOGF_INFO(AWS_LS_COMMON_GENERAL, "static: numa_num_configured_nodes() loaded");
+            } else {
+                AWS_LOGF_INFO(AWS_LS_COMMON_GENERAL, "static: numa_num_configured_nodes() failed to load");
+            }
+
+            *(void **)(&g_numa_num_possible_cpus_ptr) = dlsym(g_libnuma_handle, "numa_num_possible_cpus");
+            if (g_numa_num_possible_cpus_ptr) {
+                AWS_LOGF_INFO(AWS_LS_COMMON_GENERAL, "static: numa_num_possible_cpus() loaded");
+            } else {
+                AWS_LOGF_INFO(AWS_LS_COMMON_GENERAL, "static: numa_num_possible_cpus() failed to load");
+            }
+
+            *(void **)(&g_numa_node_of_cpu_ptr) = dlsym(g_libnuma_handle, "numa_node_of_cpu");
+            if (g_numa_node_of_cpu_ptr) {
+                AWS_LOGF_INFO(AWS_LS_COMMON_GENERAL, "static: numa_node_of_cpu() loaded");
+            } else {
+                AWS_LOGF_INFO(AWS_LS_COMMON_GENERAL, "static: numa_node_of_cpu() failed to load");
+            }
+
+        } else {
+            AWS_LOGF_INFO(AWS_LS_COMMON_GENERAL, "static: libnuma.so failed to load");
+        }
+#endif
     }
 }
 
@@ -263,6 +335,11 @@ void aws_common_library_clean_up(void) {
         s_common_library_initialized = false;
         aws_unregister_error_info(&s_list);
         aws_unregister_log_subject_info_list(&s_common_log_subject_list);
+#if !defined(_WIN32) && !defined(WIN32)
+        if (g_libnuma_handle) {
+            dlclose(g_libnuma_handle);
+        }
+#endif
     }
 }
 
