@@ -239,7 +239,13 @@ static int s_tester_init(struct tester *tester, const struct tester_options *opt
     ASSERT_SUCCESS(aws_condition_variable_init(&tester->wait_cvar));
 
     tester->event_loop_group = aws_event_loop_group_new_default(tester->alloc, 1, NULL);
-    tester->host_resolver = aws_host_resolver_new_default(tester->alloc, 8, tester->event_loop_group, NULL);
+
+    struct aws_host_resolver_default_options resolver_options = {
+        .el_group = tester->event_loop_group,
+        .max_entries = 8,
+    };
+
+    tester->host_resolver = aws_host_resolver_new_default(tester->alloc, &resolver_options);
     tester->server_bootstrap = aws_server_bootstrap_new(tester->alloc, tester->event_loop_group);
     ASSERT_NOT_NULL(tester->server_bootstrap);
 
