@@ -571,11 +571,11 @@ static void s_creation_callback_test_channel_creation_callback(
 
 static struct aws_event_loop *s_default_new_event_loop(
     struct aws_allocator *allocator,
-    aws_io_clock_fn *clock,
+    const struct aws_event_loop_options *options,
     void *user_data) {
 
     (void)user_data;
-    return aws_event_loop_new_default(allocator, clock);
+    return aws_event_loop_new_default_with_options(allocator, options);
 }
 
 static int s_statistic_test_clock_fn(uint64_t *timestamp) {
